import json
from typing import Optional


class TreeNode:
    def __init__(self, val: int, left: "TreeNode" = None, right: "TreeNode" = None):
        self.val = val
        self.left = left
        self.right = right

    @classmethod
    def deserialize(cls, s: str) -> Optional["TreeNode"]:
        res = json.loads(s)
        if not res:
            return None

        nodes = [TreeNode(val) if val is not None else None for val in res]
        root = nodes[0]

        j = 1
        for node in nodes:
            if node is not None:
                if j < len(res):
                    node.left = nodes[j]
                    j += 1
                if j < len(res):
                    node.right = nodes[j]
                    j += 1
            if j >= len(res):
                break

        return root

    def serialize(self) -> str:
        nodes = []
        queue = [self]

        while queue:
            t = queue.pop(0)
            nodes.append(t)

            if t is not None:
                queue.extend([t.left, t.right])

        while nodes and nodes[-1] is None:
            nodes.pop()

        arr = [node.val if node is not None else None for node in nodes]
        return json.dumps(arr)



import json
from typing import Optional


class ListNode:
    def __init__(self, val: int = 0, next: "ListNode" = None):
        self.val = val
        self.next = next

    @classmethod
    def deserialize(cls, s: str) -> Optional["ListNode"]:
        arr = json.loads(s)
        if not arr:
            return None
        root = ListNode(arr[0])
        node = root
        for v in arr[1:]:
            node.next = ListNode(v)
            node = node.next
        return root

    def serialize(self) -> str:
        s = ["["]
        node = self
        while node:
            s.append(str(node.val))
            node = node.next
            if node:
                s.append(",")
        s.append("]")
        return "".join(s)



import json
from typing import Any, List

from . import ListNode, TreeNode


def split_array(s: str) -> List[str]:
    s = s.strip()
    if len(s) <= 1 or s[0] != "[" or s[-1] != "]":
        raise Exception("Invalid array: " + s)

    splits = json.loads(s)
    res = [json.dumps(split) for split in splits]
    return res


def serialize(val: Any) -> str:
    if isinstance(val, int):
        return str(val)
    elif isinstance(val, float):
        return str(val)
    elif isinstance(val, str):
        return '"' + val + '"'
    elif isinstance(val, bool):
        return "true" if val else "false"
    elif isinstance(val, list):
        return "[" + ",".join(serialize(v) for v in val) + "]"
    elif isinstance(val, (ListNode, TreeNode)):
        return val.serialize()
    else:
        raise Exception("Unknown type: " + str(type(val)))


def deserialize(ty: str, s: str) -> Any:
    if ty == "int":
        return int(s)
    elif ty == "float":
        return float(s)
    elif ty == "str":
        return s[1:-1]
    elif ty == "bool":
        return s == "true"
    elif ty.startswith("List["):
        arr = []
        for v in split_array(s):
            arr.append(deserialize(ty[5:-1], v))
        return arr
    elif ty == "ListNode":
        return ListNode.deserialize(s)
    elif ty == "TreeNode":
        return TreeNode.deserialize(s)
    else:
        raise Exception("Unknown type: " + ty)
